@@ -19,7 +19,7 @@
  *           Fax: +49 7123 / 14892
  *
  *
- *  $Id: dbdimp.c 1.1 Tue, 30 Sep 1997 01:28:08 +0200 joe $
+ *  $Id: dbdimp.c,v 1.4 1999/01/05 22:30:54 joe Exp $
  */
 
 
@@ -72,23 +72,27 @@ void dbd_init(dbistate_t* dbistate) {
 
 void do_error(SV* h, int rc, char* what) {
     D_imp_xxh(h);
+    STRLEN lna;
+
     SV *errstr = DBIc_ERRSTR(imp_xxh);
     sv_setiv(DBIc_ERR(imp_xxh), (IV)rc);	/* set err early	*/
     sv_setpv(errstr, what);
     DBIh_EVENT2(h, ERROR_event, DBIc_ERR(imp_xxh), errstr);
     if (dbis->debug >= 2)
 	fprintf(DBILOGFP, "%s error %d recorded: %s\n",
-		what, rc, SvPV(errstr,na));
+		what, rc, SvPV(errstr,lna));
 }
 void do_warn(SV* h, int rc, char* what) {
     D_imp_xxh(h);
+    STRLEN lna;
+
     SV *errstr = DBIc_ERRSTR(imp_xxh);
     sv_setiv(DBIc_ERR(imp_xxh), (IV)rc);	/* set err early	*/
     sv_setpv(errstr, what);
     DBIh_EVENT2(h, WARN_event, DBIc_ERR(imp_xxh), errstr);
     if (dbis->debug >= 2)
 	fprintf(DBILOGFP, "%s warning %d recorded: %s\n",
-		what, rc, SvPV(errstr,na));
+		what, rc, SvPV(errstr,lna));
     warn("%s", what);
 }
 
@@ -123,6 +127,7 @@ void do_warn(SV* h, int rc, char* what) {
 int MyConnect(dbh_t *sock, char* unixSocket, char* host, char* port,
 	      char* user, char* password, char* dbname, imp_dbh_t *imp_dbh) {
     int portNr;
+    STRLEN lna;
 
     if (host && !*host) host = NULL;
     if (port && *port) {
@@ -178,18 +183,18 @@ int MyConnect(dbh_t *sock, char* unixSocket, char* host, char* port,
 					FALSE))  &&
 			*svp  &&  SvTRUE(*svp)) {
 		        mysql_options(*sock, MYSQL_READ_DEFAULT_GROUP,
-				      SvPV(*svp, na));
+				      SvPV(*svp, lna));
 		    }
 		    if ((svp = hv_fetch(hv, "mysql_read_default_file", 23,
 					FALSE))  &&
 			*svp  &&  SvTRUE(*svp)) {
 		        mysql_options(*sock, MYSQL_READ_DEFAULT_FILE,
-				      SvPV(*svp, na));
+				      SvPV(*svp, lna));
 		    }
+#endif /* MYSQL_VERSION_ID >= 32210 */
 		}
 	    }
 	}
-#endif /* MYSQL_VERSION_ID >= 32210 */
 #endif
 	return mysql_real_connect(*sock, host, user, password, dbname,
 				  portNr, unixSocket, 0) ?
@@ -248,7 +253,7 @@ static int _MyLogin(imp_dbh_t *imp_dbh) {
     char* user;
     char* password;
     char* unixSocket = NULL;
-    STRLEN len;
+    STRLEN len, lna;
 
     sv = DBIc_IMP_DATA(imp_dbh);
     if (!sv  ||  !SvROK(sv)) {
@@ -267,7 +272,7 @@ static int _MyLogin(imp_dbh_t *imp_dbh) {
         host = NULL;
     }
     if ((svp = hv_fetch(hv, "port", 4, FALSE))) {
-        port = SvPV(*svp, na);
+        port = SvPV(*svp, lna);
     } else {
         port = NULL;
     }
@@ -288,19 +293,19 @@ static int _MyLogin(imp_dbh_t *imp_dbh) {
         password = NULL;
     }
     if ((svp = hv_fetch(hv, "database", 8, FALSE))) {
-        dbname = SvPV(*svp, na);
+        dbname = SvPV(*svp, lna);
     } else {
         dbname = NULL;
     }
 #ifdef DBD_MYSQL
     if ((svp = hv_fetch(hv, "mysql_socket", 12, FALSE))  &&
 	*svp  &&  SvTRUE(*svp)) {
-        unixSocket = SvPV(*svp, na);
+        unixSocket = SvPV(*svp, lna);
     }
 #elif defined(IDX_TYPE)
     if ((svp = hv_fetch(hv, "msql_configfile", 15, FALSE))  &&
 	*svp  &&  SvOK(*svp)) {
-        char* cf = SvPV(*svp, na);
+        char* cf = SvPV(*svp, lna);
 	if (dbis->debug >= 2) {
 	    fprintf(DBILOGFP, "imp_dbh->MyLogin: Loading config file %s\n",
 		    cf);
@@ -683,7 +688,6 @@ SV* dbd_db_FETCH_attrib(SV* dbh, imp_dbh_t* imp_dbh, SV* keysv) {
 int dbd_st_prepare(SV* sth, imp_sth_t* imp_sth, char* statement, SV* attribs) {
     int i;
 
-
     /*
      *  Count the number of parameters
      */
@@ -696,27 +700,18 @@ int dbd_st_prepare(SV* sth, imp_sth_t* imp_sth, char* statement, SV* attribs) {
     imp_sth->cda = NULL;
     imp_sth->currow = 0;
 #if defined(DBD_MYSQL)
-    imp_sth->use_mysql_use_result = 0;
+    {
+        SV** svp = DBD_ATTRIB_GET_SVP(attribs, "mysql_use_result", 16);
+        imp_sth->use_mysql_use_result = svp && SvTRUE(*svp);
+	if (dbis->debug >= 2)
+	    fprintf(DBILOGFP, "Setting mysql_use_result to %d\n",
+		    imp_sth->use_mysql_use_result);
+    }
 #endif
     for (i = 0;  i < AV_ATTRIB_LAST;  i++) {
 	imp_sth->av_attr[i] = Nullav;
     }
 
-#if defined(DBD_MYSQL)
-    /*
-     *  Check attributes
-     */
-    if (attribs  &&  SvTYPE(attribs) == SVt_RV) {
-        SV* sv = SvRV(attribs);
-	if (SvTYPE(sv) == SVt_PVHV) {
-	    SV** svp = hv_fetch((HV*) sv, "mysql_use_result", 16, FALSE);
-	    if (svp) {
-	        imp_sth->use_mysql_use_result = SvTRUE(*svp);
-	    }
-	}
-    }
-#endif
-
     /*
      *  Allocate memory for parameters
      */
@@ -746,7 +741,11 @@ int dbd_st_prepare(SV* sth, imp_sth_t* imp_sth, char* statement, SV* attribs) {
 
 int dbd_st_internal_execute(SV* h, SV* statement, SV* attribs, int numParams,
 			    imp_sth_ph_t* params, result_t* cdaPtr,
-			    dbh_t svsock, int use_mysql_use_result) {
+			    dbh_t svsock
+#ifdef DBD_MYSQL
+			    , int use_mysql_use_result
+#endif
+			    ) {
     STRLEN slen;
     char* sbuf = SvPV(statement, slen);
     char* salloc = ParseParam(sbuf, &slen, params, numParams);
@@ -942,8 +941,11 @@ int dbd_st_execute(SV* sth, imp_sth_t* imp_sth) {
 				     DBIc_NUM_PARAMS(imp_sth),
 				     imp_sth->params,
 				     &imp_sth->cda,
-				     imp_dbh->svsock,
-				     imp_sth->use_mysql_use_result))
+				     imp_dbh->svsock
+#ifdef DBD_MYSQL
+				     ,imp_sth->use_mysql_use_result
+#endif
+				     ))
 	!= -2) {
 	if (!imp_sth->cda) {
 #if defined(DBD_MYSQL)
@@ -1185,6 +1187,14 @@ int dbd_st_STORE_attrib(SV* sth, imp_sth_t* imp_sth, SV* keysv, SV* valuesv) {
 #if defined(DBD_MYSQL)
     if (strEQ(key, "mysql_use_result")) {
         imp_sth->use_mysql_use_result = SvTRUE(valuesv);
+	result = TRUE;
+    }
+#else
+    if (DBIc_COMPAT(imp_sth)) {
+        if (strEQ(key, "OPTIMIZE")) {
+	    imp_sth->optimize = SvTRUE(valuesv);
+	    result = TRUE;
+	}
     }
 #endif
 
@@ -1294,7 +1304,9 @@ SV* dbd_st_FETCH_internal(SV* sth, int what, result_t res, int cacheit) {
 		      case FIELD_TYPE_INT24:       i = SQL_BIGINT;      break;
 		      case FIELD_TYPE_DATE:        i = SQL_DATE;        break;
 		      case FIELD_TYPE_TIME:        i = SQL_TIME;        break;
+#if defined(MYSQL_VERSION_ID)  &&  (MYSQL_VERSION_ID >= 32200)
 		      case FIELD_TYPE_NEWDATE:     i = SQL_DATE;        break;
+#endif
 		      case FIELD_TYPE_TINY_BLOB:   i = SQL_VARCHAR;     break;
 		      case FIELD_TYPE_MEDIUM_BLOB: i = SQL_LONGVARCHAR; break;
 		      case FIELD_TYPE_LONG_BLOB:   i = SQL_LONGVARCHAR; break;
@@ -1303,7 +1315,9 @@ SV* dbd_st_FETCH_internal(SV* sth, int what, result_t res, int cacheit) {
 		      case FIELD_TYPE_STRING:      i = SQL_CHAR;        break;
 		      case FIELD_TYPE_DATETIME:    i = SQL_TIMESTAMP;   break;
 		      case FIELD_TYPE_NULL:        i = SQL_VARCHAR;     break;
+#if defined(MYSQL_VERSION_ID)  &&  (MYSQL_VERSION_ID >= 32200)
 		      case FIELD_TYPE_YEAR:        i = SQL_SMALLINT;    break;
+#endif
 		      case FIELD_TYPE_ENUM:        i = SQL_CHAR;        break;
 		      case FIELD_TYPE_SET:         i = SQL_CHAR;        break;
 		      default:                     i = SQL_VARCHAR;     break;
@@ -1469,9 +1483,10 @@ SV* dbd_st_FETCH_internal(SV* sth, int what, result_t res, int cacheit) {
     dbd_st_FETCH_internal(sth, (what), imp_sth->cda, TRUE)
 
 SV* dbd_st_FETCH_attrib(SV* sth, imp_sth_t* imp_sth, SV* keysv) {
-    STRLEN(kl);
+    STRLEN kl;
     char* key = SvPV(keysv, kl);
     SV* retsv = Nullsv;
+    STRLEN lna;
 
     if (dbis->debug >= 2) {
         fprintf(DBILOGFP,
@@ -1535,6 +1550,13 @@ SV* dbd_st_FETCH_attrib(SV* sth, imp_sth_t* imp_sth, SV* keysv) {
 	    retsv = sv_2mortal(newSViv((IV) DBIc_NUM_FIELDS(imp_sth)));
 	}
 	break;
+#if defined(DBD_MSQL)
+      case 'O':
+	if (strEQ(key, "OPTIMIZE")  &&  DBIc_COMPAT(imp_sth)) {
+	    retsv = boolSV(imp_sth->optimize);	    
+	}
+	break;
+#endif
       case 'R':
 	/*
 	 * Deprecated, use 'result'
@@ -1619,7 +1641,7 @@ SV* dbd_st_FETCH_attrib(SV* sth, imp_sth_t* imp_sth, SV* keysv) {
     if (dbis->debug >= 2) {
         fprintf(DBILOGFP,
 		"    <- dbd_st_FETCH_attrib for %08lx, key %s: result %s\n",
-		(u_long) sth, key, retsv ? SvPV(retsv, na) : "NULL");
+		(u_long) sth, key, retsv ? SvPV(retsv, lna) : "NULL");
     }
 
     return retsv;
